#!/system/bin/sh

#versionCode=1

# Usage: qiku_backup.sh <0|1|>,<0|1|>,<1|2|3|4>,pkg_name|wifi
#        <0|1> for type
#        0: wifi, 1: data
#        <0|1> for operation
#        0: backup, 1: restore
#        <1|2> for path
#        1 : /data/data/com.qiku.android.cloudsync/qikubackup/data
#        2 : /data/data/com.qiku.android.cloudsync/qikubackup/localwifi
#        3 : /data/data/com.qihoo360.transfer/qikubackup/data
#        4 : /data/data/com.qihoo360.transfer/qikubackup/localwifi
#        5 : /data/data/com.qihoo360.cloneit/qikubackup/data
#        6 : /data/data/com.qihoo360.cloneit/qikubackup/localwifi
#
# Example: qiku_backup.sh 1,1,1,com.sina.weibo
#
# Output:
#        qiku.backup.status
#        <start|error|finish>
#=================================[START]=====================================

LOG_TAG="QIKU_BACKUP"

TAR="/data/data/com.qihoo360.transfer/files/busybox2 tar"
PROP_STATUS=qiku.backup.status

# output the log
logi ()
{
  #/system/bin/log -t $LOG_TAG -p i "QIKULOG:$@"
  echo "###`date`### $@">>/storage/sdcard0/360Transfer/Log/backup.log
}

# check if the arguments are wrong
if [ -z $1 ]
then
    logi "Usage: qiku_backup.sh <0|1>,<0|1>,<1|2|3|4>,pkg_name|wifi"
    exit 1
fi

setprop $PROP_STATUS start

# parse the input arguments
backup_type=${1:0:1}
operate=${1:2:1}
path=${1:4:1}
pkg_name=${1:6}

logi "$1, backup_type=$backup_type,operate=$operate, pkg_name='$pkg_name',path=$path"


# decide the place where to backup or restore
if   [ $path = "1" ]; then
    bkdir="/data/data/com.qiku.android.cloudsync/qikubackup/data"
    CallerPkg="com.qiku.android.cloudsync"
elif [ $path = "2" ]; then
    bkdir="/data/data/com.qiku.android.cloudsync/qikubackup/localwifi"
    CallerPkg="com.qiku.android.cloudsync"
elif [ $path = "3" ]; then
    bkdir="/data/data/com.qihoo360.transfer/qikubackup/data"
    CallerPkg="com.qihoo360.transfer"
elif [ $path = "4" ]; then
    bkdir="/data/data/com.qihoo360.transfer/qikubackup/localwifi"
    CallerPkg="com.qihoo360.transfer"
elif [ $path = "5" ]; then
    bkdir="/data/data/com.qihoo360.cloneit/qikubackup/data"
    CallerPkg="com.qihoo360.cloneit"
elif [ $path = "6" ]; then
    bkdir="/data/data/com.qihoo360.cloneit/qikubackup/localwifi"
    CallerPkg="com.qihoo360.cloneit"
fi

# put data in cloudsync,need to change userid
CallerPkgUserid=`ls -n -ld /data/data/$CallerPkg | busybox cut -d' ' -f 2`
CallerPkgUserlabel=`ls -d -Z /data/data/$CallerPkg | busybox tr -s [:space:] | busybox cut -d' ' -f 4`
#wifi
if [ $backup_type = "0" ]; then

    wifi_path="/data/misc/wifi/wpa_supplicant.conf"
    wifi_back="$bkdir/wpa_supplicant.conf"

    if [ $operate = "0" ]; then
        if [ ! -e $wifi_path ]; then
            logi "'$wifi_path' doesn't exist!"
            setprop $PROP_STATUS error
            exit 1
        fi

        mkdir -p $bkdir &>/dev/null

        busybox cp $wifi_path $bkdir/
        ret=$?
        logi "copy result:$ret"
        # put data in cloudsync,need to change userid
        busybox chown -R $CallerPkgUserid:$CallerPkgUserid /data/data/$CallerPkg/qikubackup
        #chcon -R $CallerPkgUserlabel /data/data/$CallerPkg/qikubackup

    elif [ $operate = "1" ]; then
        if [ ! -e $wifi_back ]; then
            logi "'$wifi_back' doesn't exist!"
            setprop $PROP_STATUS error
            exit 1
        fi

        busybox cp $wifi_back $wifi_path
        ret=$?
        logi "copy result:$ret"
        chmod 660 $wifi_path
        chown system:wifi $wifi_path
    fi

elif [ $backup_type = "1" ]; then
    # check the package exist
    if [ ! -d "/data/data/$pkg_name" ]; then
        logi "'/data/data/$pkg_name' doesn't exist!"
        setprop $PROP_STATUS error
        exit 1
    fi

    # get the user name of the app(package)
    #user=`ls -ld /data/data/$pkg_name|busybox cut -d' ' -f 2`
    userid=`ls -n -ld /data/data/$pkg_name|busybox cut -d' ' -f 2`
    userlabel=`ls -d -Z /data/data/$pkg_name | busybox tr -s [:space:] | busybox cut -d' ' -f 4`
    logi "The user ID of $pkg_name is $userid, label is $userlabel"

    # backup
    # cd /data/data/$pkg_name
    if [ $operate = "0" ]; then
        # make the directory
        mkdir -p $bkdir &>/dev/null
        # file sync
        busybox usleep 100000
        # fsync every database file
        ls -a /data/data/$pkg_name/databases | while read file
        do
            busybox fsync /data/data/$pkg_name/databases/${file}
            logi "busybox fsync /data/data/$pkg_name/databases/${file}"
            wait
        done
        sync
        # backup /storage/sdcard0/Android/data/$pkg_name/files
        if [ -e "/storage/sdcard0/Android/data/$pkg_name/files" ]; then
            logi "--internal sd data--"
            sddata="/storage/sdcard0/Android/data/$pkg_name/files"
        elif [ -e "/storage/sdcard1/Android/data/$pkg_name/files" ]; then
            # backup /storage/sdcard1/Android/data/$pkg_name/files
            logi "--external sd data--"
            sddata="/storage/sdcard1/Android/data/$pkg_name/files"
        fi
        logi "Backup sddata:$sddata"
        # backup the apk data except lib subdir
        #$TAR -czv -f $bkdir/$pkg_name.tar -X /data/data/$pkg_name/lib* /data/data/$pkg_name $sddata  #lib:no such file
        $TAR -czv -f $bkdir/$pkg_name.tar /data/data/$pkg_name $sddata
        ret=$?
        logi "backup result:$ret"
        # put data in cloudsync,need to change userid
        busybox chown -R $CallerPkgUserid:$CallerPkgUserid /data/data/$CallerPkg/qikubackup
        #chcon -R $CallerPkgUserlabel /data/data/$CallerPkg/qikubackup
    elif [ $operate = "1" ]; then
        # check if the tar file exist
        if [ ! -s "$bkdir/$pkg_name.tar" ]; then
            logi "$bkdir/$pkg_name.tar does not exist!"
            setprop $PROP_STATUS error
            exit 1
        fi
        # restore the tar file
        $TAR -zxv -f $bkdir/$pkg_name.tar data/data/$pkg_name -C /
        #check SD card insert and restore
        insertsd=`mount | grep sdcard1 | busybox cut -d ' ' -f 2`
        if [ "$insertsd" != "" ]; then
            logi "sd card insert."
            sddatapath=`$TAR -tv -f $bkdir/$pkg_name.tar| grep "storage/sdcard1/Android/data/$pkg_name/files"`
            if [ "$sddatapath" != "" ]; then
                logi "Restore external data."
                $TAR -zxv -f $bkdir/$pkg_name.tar storage/sdcard1/Android/data/$pkg_name/files -C /
            else
                #check internal and restore
                intdatapath=`$TAR -tv -f $bkdir/$pkg_name.tar| grep "storage/sdcard0/Android/data/$pkg_name/files"`
                if [ "$intdatapath" != "" ]; then
                    logi "Restore internal data."
                    $TAR -zxv -f $bkdir/$pkg_name.tar storage/sdcard0/Android/data/$pkg_name/files -C /
                fi
            fi
        else
            #check internal and restore
            logi "sd card not insert."
            intdatapath=`$TAR -tv -f $bkdir/$pkg_name.tar| grep "storage/sdcard0/Android/data/$pkg_name/files"`
            if [ "$intdatapath" != "" ]; then
                logi "Restore internal data."
                $TAR -zxv -f $bkdir/$pkg_name.tar storage/sdcard0/Android/data/$pkg_name/files -C /
            fi
        fi
        ret=$?
        logi "restore result:$ret"
        # chown every file if need
        chown $userid:$userid /data/data/$pkg_name
        ls -a /data/data/$pkg_name | while read file1
        do
            if [ "${file1}" != "lib" ]; then
                busybox chown -R $userid:$userid /data/data/$pkg_name/${file1}
                # add for selinux label by xuzhike@qiku.com
                #chcon -R $userlabel /data/data/$pkg_name/${file1}
                #logi "chown $userid:$userid /data/data/$pkg_name/${file1}"
            fi
        done
    fi

fi

# tell the backup apk we finish
setprop $PROP_STATUS finish

exit 0
#===================================[END]=====================================
